from typing import Union

from _pytest.config import ExitCode
from _pytest.main import Session


def pytest_zulip_create_content(session: Session, exitstatus: Union[int, ExitCode]) -> str:
    """ Called to create content """



import os
from typing import Union

import pytest
from _pytest.config import ExitCode, Config
from _pytest.main import Session
from _pytest.terminal import TerminalReporter

from pytest_zulip.zulip import send_message


def pytest_addhooks(pluginmanager):
    from . import hooks

    pluginmanager.add_hookspecs(hooks)


def pytest_addoption(parser):
    parser.addoption("--notify", action="store_true", help="Notify via Zulip")


def pytest_sessionfinish(session: Session, exitstatus: Union[int, ExitCode]):
    if session.config.getoption("--notify"):
        send_message(session, exitstatus)


@pytest.hookimpl(hookwrapper=True)
def pytest_terminal_summary(
    terminalreporter: TerminalReporter, exitstatus: int, config: Config
):
    yield
    # special check for pytest-xdist plugin, cause we do not want to send report for each worker.
    if hasattr(terminalreporter.config, "workerinput"):
        return
    if config.getoption("--notify"):
        terminalreporter.write_sep("-", "notification sent on Zulip")


def pytest_configure(config):
    if config.getoption("--notify") and (
        not os.environ.get("ZULIP_URL")
        or not os.environ.get("ZULIP_STREAM")
        or not os.environ.get("ZULIP_TOPIC")
        or not os.environ.get("ZULIP_BOT_EMAIL_ADDRESS")
        or not os.environ.get("ZULIP_BOT_API_KEY")
    ):
        raise Exception("You must set the environment variables for Zulip plugin")



import os
from typing import Union

import requests
from _pytest.config import ExitCode
from _pytest.main import Session
from _pytest.terminal import TerminalReporter


def send_message(session: Session, exitstatus: Union[int, ExitCode]):
    # https://zulip.com/api/send-message
    # https://zulip.com/help/format-your-message-using-markdown#status-messages
    status = "succeeded"
    if exitstatus != 0:
        status = "failed"
    topic = f"{os.environ.get('ZULIP_TOPIC')} {status}"
    content = session.config.hook.pytest_zulip_create_content(session=session, exitstatus=exitstatus)
    if not content:
        reporter = session.config.pluginmanager.get_plugin('terminalreporter')
        reporter: TerminalReporter
        content = (
            f"# Test {session.nodeid} {status}\n"
            f"Passed={len(reporter.stats.get('passed', []))} Failed={len(reporter.stats.get('failed', []))} "
            f"Skipped={len(reporter.stats.get('skipped', []))} Error={len(reporter.stats.get('error', []))}"
        )
    payload = {
        "type": "stream",
        "to": os.environ.get("ZULIP_STREAM"),
        "topic": topic,
        "content": _trim_string(content, 1000),
    }
    requests.post(
        url=os.environ.get("ZULIP_URL"),
        auth=(
            os.environ.get("ZULIP_BOT_EMAIL_ADDRESS"),
            os.environ.get("ZULIP_BOT_API_KEY"),
        ),
        data=payload,
    )


def _trim_string(s: str, limit: int, ellipsis_char="…") -> str:
    s = s.strip()
    sb = s.encode("utf-8")
    limit = limit - len(ellipsis_char.encode("utf-8"))
    if len(sb) > limit:
        sb_limited = sb[:limit]
        return sb_limited.decode("utf-8").strip() + ellipsis_char
    return s



#!/usr/bin/env python
import glob

from setuptools import setup, find_packages

from pytest_zulip import __version__

data_files = []
directories = glob.glob("pytest_zulip/")
for directory in directories:
    files = glob.glob(directory + "*/*")
    data_files.append((directory, files))


setup(
    name="pytest-zulip",
    version=__version__,
    url="https://github.com/treussart/pytest-zulip",
    license="BSD",
    description="Send report on Zulip",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    keywords="py.test pytest zulip report",
    author="mtreussart",
    author_email="matthieu@treussart.com",
    packages=find_packages(
        include=["pytest_zulip"],
    ),
    data_files=data_files,
    include_package_data=True,
    python_requires=">=3.7",
    install_requires=["requests", "pytest"],
    entry_points={"pytest11": ["zulip = pytest_zulip.plugin"]},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Pytest",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Topic :: Software Development :: Quality Assurance",
        "Topic :: Software Development :: Testing",
        "Topic :: Utilities",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3 :: Only",
    ],
)
